/**          _     _            _           ____  
 * __      _(_) __| | __ _  ___| |_  __   _|___ \ 
 * \ \ /\ / / |/ _` |/ _` |/ _ \ __| \ \ / / __) |
 *  \ V  V /| | (_| | (_| |  __/ |_   \ V / / __/ 
 *   \_/\_/ |_|\__,_|\__, |\___|\__|___\_/ |_____|
 *                   |___/     © npoxop.sochi 2022
 */
(function(){
	'use strict';
	
	$.fancybox.defaults.touch = false;
	
	$.B24 = {
		requestDataExt: false,
		
		imageOnload: function(){
			return {
				restrict: 'A',
				link: function(scope, element, attrs){
					element.bind('load', function(e){
						scope.$apply(function(){
							scope.$eval(attrs.imageOnload);
						});
						e.preventDefault();
					});
				},
			};
		},
		
		masked: function(){
			return {
				restrict: 'A',
				require: '?ngModel',
				link: function(scope, element, attrs, ngModelCtrl){
					element.mask(attrs.format, {
						completed: function(){
							ngModelCtrl.$setViewValue(element.val());  
							scope.$apply();
						},
					});
				},
			};
		},
		
		numbersOnly: function(){
			return {
				require: 'ngModel',
				link: function(scope, element, attrs, modelCtrl){
					modelCtrl.$parsers.push(function(inputValue){
						if (inputValue == undefined) return '';
						var onlyNumeric = inputValue.replace(/[^0-9]/g, '');
						if (onlyNumeric != inputValue) {
							modelCtrl.$setViewValue(onlyNumeric);
							modelCtrl.$render();
						}
						return onlyNumeric;
					});
				},
			};
		},
		
		local: function(params){
			let local = 'en_EN.json';
			switch (params.lng) {
				case 'ru':
					local = 'ru_RU.json';
				break;
				case 'en':
					local = 'en_EN.json';
				break;
				case 'am':
					local = 'am_AM.json';
				break;
			}
			return local;
		},
		
		ext: function($location){
			let get = $location.search();
			
			return $.extend({
				frontendId: -1,															// fid
				token: 'none',															// token
				zone: 'test',																// режим, по умолчанию - тестовый
				id: -1,																			// идентификатор события
				cityId: -1,																	// идентификатор города
				venueId: -1,																// идентификатор площадки, не обязательно
				kindId: -1,																	// идентификатор типа события, не обязательно
				success: 'https://bilw.ru/pay_s.html',			// страница удачной оплаты
				fail: 'https://bilw.ru/pay_f.html',					// страница неудачной оплаты
				agr: 'https://bilw.ru/agreement.html',			// страница оферты
				bank: undefined,														// открывать ли страницу банка в новой вкладке, по умолчанию - нет
				goods: undefined,														// билеты или товар, по умолчанию - билеты
				orpheo: undefined,													// проверять ли билеты на наличие аудиогида, по умолчанию - нет
				lng: 'ru',																	// язык, по умолчанию - русский
				event: undefined,														// не показывать список событий
				devel: undefined,														// режим отладки
			}, get);
		},
				
		Config: function(S, G, LS){
			let C = $.B24.ext(G);
			
			let renameKeys = function(obj, to){
				const keys = Object.keys(obj).map(key => {
					const i = to[key] || key;
					return {[i]: obj[key]};
				});
				return Object.assign({}, ...keys);
			};
			C = renameKeys(C, {
				frontendId: 'fid',
				id: 'actionId',
				success: 'successUrl',
				fail: 'failUrl',
			});
			
			if (typeof LS.userId !== 'undefined') {
				C.userId = LS.userId;
			}
			if (typeof LS.sessionId !== 'undefined') {
				C.sessionId = LS.sessionId;
			}
			
			C.terms = {
				fullName: {required: false, value: ''},
				phone: {required: false, value: ''},
			};
			
			S.config = C;
			
			S.cart = {
				timeout: {
					function: false,
					time: false,
					format: false,
				},
				withPlace: {
					data: [],
					count: 0,
					show: false,
					noResults: false,
				},
				entries: {
					data: {},
					tickets: 0,
					show: false,
				},
				totalCharge: 0,
				totalSum: 0,
			}
			
			S.miniBasket = {
				count: 0,
				show: false
			};
			
			S.fullBasket = {
				filled: false,
				loading: false,
				show: false
			}
		},
		
		Translate: function(S, R){
			let L = 'en_EN';
			switch (S.config.lng) {
				case 'ru':
					L = 'ru_RU';
				break;
				case 'am':
					L = 'am_AM';
				break;
			}
			S.config.local = L;
			let path = {local: L};
			let i = 1;
			for (let p of window.location.pathname.split('/')) {
				if (p && p != S.path) {
					path['path' + i] = p;
					i++;
				}
			}
			let request = R.GetLocalization(path);
			if (request) {
				request.$promise.then(function(response){
					let localizationCorrection = function(localization, root = null){
						for (let name in localization) {
							if (!name.match(/^\$+/)) {
								if (typeof localization[name] == 'object') {
									let type = localizationCorrection(localization[name], name);
									if (type) localization[name] = type;
								} else {
									let type = S.config.goods ? '_goods' : '_tickets';
									if (name == type) return localization[name];
								}
							}
						}
						return false;
					};
					localizationCorrection(response);
					S.translate = response;
				}, function(response){console.log(response)});
			}
		},
		
		datepicker: function(){
			return {
				restrict: 'A',
				require: 'ngModel',
				link: function(scope, elem, attrs, ngModelCtrl){
					var updateModel = function(dateText){
						scope.$apply(function(){
							ngModelCtrl.$setViewValue(dateText);
						});
					};
					setTimeout(function(){
						let L = scope.translate;
						var options = {
							monthNames: [
								L.months['01'],
								L.months['02'],
								L.months['03'],
								L.months['04'],
								L.months['05'],
								L.months['06'],
								L.months['07'],
								L.months['08'],
								L.months['09'],
								L.months['10'],
								L.months['11'],
								L.months['12']
							],
							dayNamesMin: L.days,
							firstDay: 1,
							inline: true,
							dateFormat: 'dd.mm.yy',
							minDate: new Date(),
							onSelect: function(dateText){
								updateModel(dateText);
							},
						};
						elem.datepicker(options);
					}, 2000);
				},
			};
		},
		
		Request: function($resource, $location){

			let params = $.B24.ext($location),
			//let params = $location.search(),
					requestUrl = params.lng == 'ru' ? 'https://api.bil24.pro:' : 'https://api.tixgear.com:';
			
			switch (params.zone) {
				case 'real':
					requestUrl += 443;
				break;
				case 'test':
					requestUrl += 1240;
				break;
				default:
					requestUrl += params.zone;
			}
			requestUrl += '/json';
			
			let requestData = {
				fid: '@fid',
				token: '@token',
				versionCode: '@versionCode',
				command: '@command',
				phone: '@phone',
				fullName: '@fullName',
				userId: '@userId',
				sessionId: '@sessionId',
			};
			
			if ($.B24.requestDataExt) {
				requestData = $.extend({
					categoryQuantityMap: '@categoryQuantityMap',
					code: '@code',
					email: '@email',
					seatList: '@seatList',
					type: '@type',
				}, requestData);
			}
			
			return $resource(requestUrl, JSON.stringify(requestData), {
				GetRequest: {
					method: 'POST',
					headers: {'content-type': 'application/x-www-form-urlencoded; charset=UTF-8'},
					isArray: false,
				},
				GetLocalization: {
					method: 'GET',
					url: '/:path1/:path2/:path3/:path4/:path5/:path6/:path7/:path8/:path9/:path10/localization/:local.json',
					isArray: false,
				},
				GetOrpheoAuth: {
					method: 'POST',
					url: 'https://tix.am/api/orpheo/Auth/json',
					headers: {'content-type': 'application/x-www-form-urlencoded; charset=UTF-8'},
					isArray: false,
				},
				GetOrpheoGateway: {
					method: 'POST',
					url: 'https://tix.am/api/orpheo/Gateway/json',
					headers: {'content-type': 'application/x-www-form-urlencoded; charset=UTF-8'},
					isArray: false,
				}
			});
		},
		
		Message: function(obj){
			let messages = [];
			for (let key in obj) {
				let message = obj[key];
				if (typeof message == 'number' || typeof message == 'string') {
					messages.push(key + ': ' + message);
				}
			}
			return messages;
		},
		
		Auth: function(R, S, growl, LS){
			let auth = function(){
				let request = R.GetRequest($.B24.params({command: 'AUTH'}, S.config, LS));
				if (request) {
					request.$promise.then(function(response){
						if (response.resultCode != 0) {
							let message = response.description;
							if (S.config.devel) message = $.B24.Message(response).join('<br/>');
							growl.error(message);
							
							LS.$reset();
						} else {
							S.auth = true;
						}
					}, function(response){console.log(response)});
				}
			};
			
			if (typeof LS.userId === 'undefined' || typeof LS.sessionId === 'undefined') {
				let request = R.GetRequest($.B24.params({command: 'AUTH'}, S.config));
				if (request) {
					request.$promise.then(function(response){
						if (response.resultCode != 0) {
							let message = response.description;
							if (S.config.devel) message = $.B24.Message(response).join('<br/>');
							growl.error(message);
							
						} else {
							LS.userId = response.userId;
							LS.sessionId = response.sessionId;
							auth();
						}
					}, function(response){console.log(response)});
				}
			} else {
				auth();
			}
		},
		
		OpenAuth: function(R, S, growl, LS){
			$.fancybox.open({
				src: '#auth-popup',
				beforeShow: function(){
					let request = R.GetRequest($.B24.params({command: 'GET_EMAIL'}, S.config, LS));
					if (request) {
						request.$promise.then(function(response){
							if (response.resultCode != 0) {
								let message = response.description;
								if (S.config.devel) message = $.B24.Message(response).join('<br/>');
								growl.error(message);
							
							} else {
								S.config.email = response.email;
								S.formError = false;
								if (S.config.email) {
									S.formMessage = S.translate.email.message.authorize + ' ' + S.config.email;
								} else {
									S.formMessage = S.translate.email.message.anonimus;
								}
								if (typeof S.email !== 'undefined') {
									if (S.email == S.config.email) {
										S.formMessage = S.translate.email.confirmed + ' ' + S.email;
									} else {
										S.formMessage = S.translate.email.form.bind.send + ' ' + S.email;
									}
								}
							}
						});
					}
				}
			});
		},
		
		type: function($scope){
			let localizationCorrection = function(localization, root = null){
				for (let name in localization) {
					if (!name.match(/^\$+/)) {
						if (typeof localization[name] == 'object') {
							let type = localizationCorrection(localization[name], name);
							if (type) localization[name] = type;
						} else {
							let type = $scope.config.goods ? '_goods' : '_tickets';
							if (name == type) return localization[name];
						}
					}
				}
				return false;
			};
			localizationCorrection($scope.localization);
		},
		
		localization: function(config, params){
			if (typeof params.frontendId !== 'undefined') config.fid = params.frontendId;
			if (typeof params.token !== 'undefined') config.token = params.token;
			if (typeof params.id !== 'undefined') config.actionId = params.id;
			if (typeof params.cityId !== 'undefined') config.cityId = params.cityId;
			if (typeof params.success !== 'undefined' && params.success != 'appsuccess') {
				config.successUrl = params.success;
			} else {
				config.successUrl = 'https://bilw.ru/TAC/pay_s.html';
			}
			if (typeof params.fail !== 'undefined' && params.fail != 'appfail') {
				config.failUrl = params.fail;
			} else {
				config.failUrl = 'https://bilw.ru/TAC/pay_f.html';
			}
			if (typeof params.zone !== 'undefined') config.zone = params.zone;
			if (typeof params.lng !== 'undefined') config.lng = params.lng;
		},
		
		params: function(params, config, localStorage = false){
			params = $.extend({
				fid: config.fid,
				token: config.token,
				versionCode: config.versionCode,
				locale: config.lng,
				versionCode: '1.0',
			}, params);
			if (localStorage) {
				params = $.extend({
					userId: localStorage.userId,
					sessionId: localStorage.sessionId,
				}, params);
			}
			return params;
		},
		
		href: function(config, page, href){
			href = $.extend({
				frontendId: config.fid,
				token: config.token,
				zone: config.zone,
				cityId: config.cityId,
				success: config.successUrl,
				fail: config.failUrl,
				agr: config.agr,
				lng: config.lng,
			}, href);
			
			let array = [];
			for (let i in href) array.push(i + '=' + href[i]);
			
			if (config.venueId >= 0) array.push('venueId=' + config.venueId);
			if (config.kindId >= 0) array.push('kindId=' + config.kindId);
			if (config.goods >= 0) array.push('goods=' + config.goods);
			if (config.orpheo >= 0) array.push('orpheo=' + config.orpheo);
			if (config.bank >= 0) array.push('bank=' + config.bank);
			if (config.event >= 0) array.push('event=' + config.event);
			if (config.devel >= 0) array.push('devel=' + config.devel);
			
			return page + '#/?' + array.join('&');
		},
		
		orpheo: function(params = false, config = false, localStorage = false){
			params = $.extend({
				login: 'tixam',
				token: '632ac2-bbd8ab-3fac3a',
			}, params);
			
			params = $.extend({
				b24_fid: config.fid,
				b24_token: config.token,
				language: config.lng,
			}, params);
			
			if (localStorage) {
				params = $.extend({
					key: localStorage.orpheoKey,
					b24_userId: localStorage.userId,
					b24_sessionId: localStorage.sessionId,
				}, params);
			}
			return params;
		},
		
		currency: {
			RUB: "\u20BD",
			AMD: "\u0534",
			USD: "\u0024",
			EUR: "\u20AC",
		},
	};
})();
